@@ -10,23 +10,24 @@
 //!
 //! ```ignore
 //! // One-time code:
-//! let client = ::reqwest::blocking::Client::new();
+//! let client = reqwest::blocking::Client::new();
 //!
 //! let mut realm_credentials = HashMap::new();
 //! realm_credentials.insert(
 //!     "Fake Realm".into(),
-//!     UsernamePasswordCredential::new("username", "password"),
+//!     Arc::new(UsernamePasswordCredential::new("username", "password")),
 //! );
-//! let credential = HttpRealmCredentials::new(realm_credentials);
+//! let credential = Arc::new(HttpRealmCredentials::new(realm_credentials));
 //!
 //! // Per-request code:
-//! let mut scheme = HttpAuthentication::new(&credential);
+//! let mut authentication = HttpAuthentication::new(credential);
+//!
 //! let response = loop {
-//!     while let Some(auth_step) = scheme.step() {
+//!     while let Some(auth_step) = authentication.step()? {
 //!         match auth_step {
 //!             AuthenticationStep::Request(request) => {
 //!                 let auth_response = client.execute(request);
-//!                 scheme.respond(auth_response);
+//!                 authentication.respond(auth_response);
 //!             }
 //!             AuthenticationStep::WaitFor(duration) => {
 //!                 std::thread::sleep(duration);
@@ -34,14 +35,12 @@
 //!         }
 //!     }
 //!
-//!     let request = client
+//!     let response = client
 //!         .get("https://httpbin.org/basic-auth/username/password")
-//!         .with_authentication(&scheme)
-//!         .build()?;
+//!         .with_authentication(&authentication)?
+//!         .send()?;
 //!
-//!     let response = client.execute(request)?;
-//!
-//!     if scheme.has_completed(&response)? {
+//!     if authentication.has_completed(&response)? {
 //!         break response;
 //!     }
 //! };
@@ -50,7 +49,7 @@
 //! The creation of the request takes place inside a loop. First, the authentication protocol is given an opportunity to perform any third-party calls using `step()`.
 //! HTTP Basic authentication does not use this, but it can be used, for example, to refresh an expired OAuth2 access token.
 //!
-//! The request is created using a standard `reqwest` RequestBuilder, using a new `with_authentication()` method to modify the request for the authentication protocol.
+//! The request is created using a standard `reqwest::RequestBuilder`, using a new `with_authentication()` method to modify the request for the authentication protocol.
 //! For HTTP authentication, the first iteration makes no change to the request.
 //!
 //! The request is sent and a response is received.  For HTTP authentication, this returns a `401 Unauthorized` response.
@@ -66,7 +65,6 @@ use std::time::Duration;
 use thiserror::Error;
 
 pub mod credential;
-mod sensitive;
 
 #[cfg(feature = "hyper")]
 pub mod hyper;
@@ -79,10 +77,24 @@ pub enum AuthenticError {
     #[error("Hyper error")]
     Hyper(#[from] ::hyper::Error),
 
+    #[error("Invalid header value")]
+    InvalidHeaderValue(#[from] ::http::header::InvalidHeaderValue),
+
+    #[cfg(feature = "reqwest")]
+    #[error("Invalid Header Name")]
+    ReqwestInvalidHeaderName(#[from] ::reqwest::header::InvalidHeaderName),
+
     #[cfg(feature = "reqwest")]
     #[error("Reqwest error")]
     Reqwest(#[from] ::reqwest::Error),
 
+    #[cfg(feature = "jwt")]
+    #[error("JWT encoding error")]
+    JsonWebToken(#[from] ::jsonwebtoken::errors::Error),
+
+    #[error("System time error")]
+    SystemTime(#[from] ::std::time::SystemTimeError),
+
     #[error("No credentials found for realm {0:?}")]
     UnknownRealm(String),
 
@@ -95,27 +107,13 @@ pub enum AuthenticationStep<Request> {
     WaitFor(Duration),
 }
 
-pub trait AuthenticationProcess {
-    fn auth_step<Request>(&mut self) -> Option<AuthenticationStep<Request>> {
-        None
-    }
-
-    fn auth_response<Response, Error>(
-        &mut self,
-        #[allow(unused_variables)] response: Result<Response, Error>,
-    ) {
-        panic!("Unexpected auth response");
-    }
-}
-
 pub trait AuthenticationProtocol {
-    type Builder;
     type Request;
     type Response;
     type Error;
 
-    fn step(&mut self) -> Option<AuthenticationStep<Self::Request>> {
-        None
+    fn step(&self) -> Result<Option<AuthenticationStep<Self::Request>>, AuthenticError> {
+        Ok(None)
     }
 
     fn respond(
@@ -125,10 +123,6 @@ pub trait AuthenticationProtocol {
         unimplemented!();
     }
 
-    fn configure(&self, builder: Self::Builder) -> Self::Builder {
-        builder
-    }
-
     fn has_completed(
         &mut self,
         #[allow(unused_variables)] response: &Self::Response,
@@ -137,47 +131,33 @@ pub trait AuthenticationProtocol {
     }
 }
 
-// Allow request builder authentication to use fluent model.
-pub trait WithAuthentication<Request, Response, Error>
-where
-    Self: Sized,
-{
-    #[must_use]
-    fn with_authentication(
-        self,
-        protocol: &dyn AuthenticationProtocol<
-            Builder = Self,
-            Request = Request,
-            Response = Response,
-            Error = Error,
-        >,
-    ) -> Self {
-        protocol.configure(self)
+pub trait AuthenticationProtocolConfigure<Builder> {
+    fn configure(&self, builder: Builder) -> Result<Builder, AuthenticError> {
+        Ok(builder)
     }
 }
 
-#[cfg(feature = "hyper")]
-impl
-    WithAuthentication<
-        ::hyper::Request<::hyper::Body>,
-        ::hyper::Response<::hyper::Body>,
-        ::hyper::Error,
-    > for http::request::Builder
+// Allow request builder authentication to use fluent model.
+pub trait WithAuthentication
+where
+    Self: Sized,
 {
+    fn with_authentication<Configure>(self, protocol: &Configure) -> Result<Self, AuthenticError>
+    where
+        Configure: AuthenticationProtocolConfigure<Self>,
+    {
+        protocol.configure(self)
+    }
 }
+#[cfg(feature = "hyper")]
+impl WithAuthentication for http::request::Builder {}
 
 #[cfg(feature = "reqwest")]
-impl WithAuthentication<::reqwest::Request, ::reqwest::Response, ::reqwest::Error>
-    for ::reqwest::RequestBuilder
-{
-}
+impl WithAuthentication for ::reqwest::RequestBuilder {}
+#[cfg(feature = "reqwest")]
+impl WithAuthentication for ::reqwest::Request {}
 
 #[cfg(feature = "reqwest_blocking")]
-impl
-    WithAuthentication<
-        ::reqwest::blocking::Request,
-        ::reqwest::blocking::Response,
-        ::reqwest::Error,
-    > for ::reqwest::blocking::RequestBuilder
-{
-}
+impl WithAuthentication for ::reqwest::blocking::RequestBuilder {}
+#[cfg(feature = "reqwest_blocking")]
+impl WithAuthentication for ::reqwest::blocking::Request {}
